@@ -1,8 +1,6 @@
 /******************************************************************************/
 
 #include <bigstatsr/BMAcc.h>
-#include <bigstatsr/utils.h>
-#include <Rcpp.h>
 
 using namespace Rcpp;
 using std::size_t;
@@ -15,14 +13,14 @@ void replaceSNP(Environment BM,
                 const IntegerVector& rowInd,
                 const IntegerVector& colInd) {
 
-  XPtr<FBM> xpBM = BM["address"];
-  BMAcc<unsigned char> macc(xpBM);
+  XPtr<FBM_RW> xpBM = BM["address_rw"];
+  BMAcc_RW<unsigned char> macc(xpBM);
 
   XPtr<FBM> xpBM2 = BM2["address"];
   SubBMAcc<unsigned char> macc2(xpBM2, rowInd - 1, colInd - 1);
 
-  myassert(macc.nrow() == macc2.nrow(), ERROR_DIM);
-  myassert(macc.ncol() == macc2.ncol(), ERROR_DIM);
+  myassert_size(macc.nrow(), macc2.nrow());
+  myassert_size(macc.ncol(), macc2.ncol());
 
   for (size_t j = 0; j < macc.ncol(); j++)
     for (size_t i = 0; i < macc.nrow(); i++)
